<?php
/*
Plugin Name: Football Writer
Plugin URI: http://www.footballwriter.ai
Description: Integrates with footballwriter.ai content creation assistant to create articles and blog posts
Version: 1.0
Author: Football Writer
Author URI: http://www.footballwriter.ai
*/

defined('ABSPATH') or die('No script kiddies please!');

const FW_API_URL = "https://api.footballwriter.ai";
const FW_URL = "https://app.footballwriter.ai";

require_once(plugin_dir_path(__FILE__) . 'lib/parsedown/Parsedown.php');

function fw_activate_plugin() {
    add_option('fw_api_token', '');
    add_option('fw_api_secret', '');
}

function fw_deactivate_plugin() {
    delete_option('fw_api_token');
    delete_option('fw_api_secret');
}

register_activation_hook(__FILE__, 'fw_activate_plugin');
register_deactivation_hook(__FILE__, 'fw_deactivate_plugin');

add_action('admin_menu', 'fw_admin_menu');

function fw_enqueue_scripts($hook) {
    // Only load on specific admin pages, e.g., settings page
    if ($hook != 'football-writer_page_football-writer-create-article' && $hook != 'toplevel_page_football-writer-settings') {
        return;
    }


    wp_enqueue_style('fw-admin-style-bs', plugin_dir_url(__FILE__) . 'css/bootstrap.min.css');
    wp_enqueue_style('fw-admin-style', plugin_dir_url(__FILE__) . 'css/fw-style.css');

    wp_enqueue_script('fw-script', plugin_dir_url(__FILE__) . 'js/fw-script.js', array('jquery'), null, true);

    // Pass AJAX URL to script
    wp_localize_script('fw-script', 'fwAjax', array(
        'ajax_url' => admin_url('admin-ajax.php'),
        'security' => wp_create_nonce('fw_nonce')
    ));
}
add_action('admin_enqueue_scripts', 'fw_enqueue_scripts');

function fw_ajax_handler() {
    // Check for nonce security, if needed
    check_ajax_referer('fw_nonce', 'security');

    // Process the data
    $data = $_POST['data'] ?? [];
    $token = get_option('fw_api_token');
    if (!$data || !$data["leagueId"]) {
        wp_send_json_error("API Error", 403);
    }

    $curl = curl_init();

    $apiData = [];
    $apiData["apiToken"] = $token;
    $apiData["leagueId"]  = $data["leagueId"];
    curl_setopt_array($curl, array(
        CURLOPT_URL => FW_API_URL.'/integrationLeagueMatches',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS => json_encode($apiData),
        CURLOPT_HTTPHEADER => array(
            'Content-Type: application/json'
        ),
    ));

    $response = curl_exec($curl);
    $apiResponse = json_decode($response, true);
    curl_close($curl);

    wp_send_json_success($apiResponse);
}
add_action('wp_ajax_fw_action', 'fw_ajax_handler');

function validateHmac($dataHash, $receivedSignature, $secret) {
    // JSON encode the savedArticle object
    // Generate the HMAC using the same algorithm (MD5) and the secret key
    $calculatedSignature = hash_hmac('md5', $dataHash, $secret);

    // Compare the calculated signature with the received signature
    return hash_equals($calculatedSignature, $receivedSignature);
}
function fw_ajax_create_handler() {
    // Check for nonce security, if needed
    check_ajax_referer('fw_nonce', 'security');
    $token = get_option('fw_api_token');
    // Process the data
    $data = $_POST['data'] ?? [];
    $draft_content = array(
        '<!-- wp:paragraph --><p>Your article is currently in preparation and will be displayed here upon completion. <br/>Thank you for your patience.</p><!-- /wp:paragraph -->',
        '<!-- wp:paragraph --><p>Feel free to navigate away from this page if needed—your article\'s creation will continue uninterrupted. <br /> We will notify you by email once your article is ready.</p><!-- /wp:paragraph -->',
        '<!-- wp:paragraph --><p><i>Articles in languages other than English might take longer time to generate.</i></p><!-- /wp:paragraph -->',
        '<!-- wp:image {"id":123} --><figure class="wp-block-image"><img src="' . FW_URL . '/please-wait.png" alt="Please Wait"/></figure><!-- /wp:image -->'
    );
    $post_data = array(
        'post_title'    => $data['title'],
        'post_content'  => implode("\r\n", $draft_content),
        'post_status'   => 'draft',
        'post_author'   => 1,
        'post_type'     => 'post',
    );

    $post_id = wp_insert_post($post_data);

    $apiOptions = [
        "apiToken" => $token,
        "returnUrl" => admin_url('admin.php?fw_webhook=1&wp_post_id='.$post_id),
        "options" => $data
    ];
    $curl = curl_init();

    curl_setopt_array($curl, array(
        CURLOPT_URL => FW_API_URL . '/integration',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS => json_encode($apiOptions),
        CURLOPT_HTTPHEADER => array(
            'Content-Type: application/json'
        ),
    ));
    $response = curl_exec($curl);
    $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    $response = json_decode($response, true);
    if($response['status'] == 'generating') {
        wp_send_json_success(get_edit_post_link($post_id));
    } else {
        wp_send_json_error("something went wrong", $httpcode);
    }
}
add_action('wp_ajax_fw_create_action', 'fw_ajax_create_handler');
function fw_admin_menu() {
    add_menu_page(
        'Football Writer | Settings',
        'Football Writer',
        'manage_options',
        'football-writer-settings',
        'fw_settings_page'
    );
    add_submenu_page(
        'football-writer-settings',
        'Football Writer | Settings',
        'Settings',
        'manage_options',
        'football-writer-settings',
        'fw_settings_page'
    );
    add_submenu_page(
        'football-writer-settings',
        'Football Writer | Create Article',
        'Create Article',
        'manage_options',
        'football-writer-create-article',
        'fw_create_article_page'
    );
}

function fw_create_article_page() {
    $curl = curl_init();
    $token = get_option('fw_api_token');

    $apiOptions = [];
    $apiOptions["apiToken"] = $token;
    curl_setopt_array($curl, array(
        CURLOPT_URL => FW_API_URL.'/integrationLeagues',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS =>json_encode($apiOptions),
        CURLOPT_HTTPHEADER => array(
            'Content-Type: application/json'
        ),
    ));

    $response = curl_exec($curl);

    curl_close($curl);
    $leagues = json_decode($response);
    ?>
    <div class="container-fluid mt-3 result-message-wrapper">
        <div class="alert alert-success d-none" role="alert">
            <h4 class="alert-heading">Well done!</h4>
            <p>Your article request has been sent successfully.</p>
            <p>A draft post placeholder for the article is now created, Once Football Writer generates the article the draft post will be updated with the article content.</p>
            <p>Football Writer might take 2-3 minutes to generate the article for you, thanks for your patience.</p>
            <hr>
            <p class="mb-2">You can view the draft post here <a href="#" class="view-article-link" target="_blank">here</a>.</p>
            <p class="mb-0">Or you can generate another article <a href="#" class="create-article-link">here</a>.</p>
        </div>
        <div class="alert alert-danger d-none" role="alert">
            <h4 class="alert-heading">Oh snap!</h4>
            <p>Something went wrong, please try again.</p>
        </div>
        <div class="alert alert-danger d-none payment" role="alert">
            <h4 class="alert-heading">Out of credit</h4>
            <p>Your account is out of credit.</p>
            <p>Go to <a href="https://app.footballwriter.ai/pricing" target="_blank">Football Writer</a> account to purchase extra credit.</p>
        </div>
    </div>
    <div class="container-fluid mt-3 generate-wrapper">
        <h1 class="mb-5">Football Writer Create Article</h1>
        <div id="create-article-wrapper">
            <h3>1 - Select League</h3>
            <div class="leagues-wrapper">
                <?php
                foreach ($leagues as $league) {
                    ?>
                    <div class="league-wrapper">
                        <button type="button" class="league" data-leagueId="<?= $league->id ?>" data-name="<?= $league->name ?>">
                            <img src="<?= FW_URL ?>/<?= $league->logo ?>"  alt="<?= $league->name ?>"/>
                            <div class="name"><?= $league->name ?></div>
                        </button>
                    </div>
                    <?php
                }
                ?>
            </div>
            <div id="matches">

            </div>
            <div id="generate-article" class="d-none">
                <hr>
                <h3>3 - Generate</h3>
                <div class="mb-3">
                    <label for="title" class="form-label">Article Title</label>
                    <input type="text" class="form-control" id="title" placeholder="Article title">
                </div>
                <div class="mb-3">
                    <label for="SEOKeyword" class="form-label">SEO focus keyword</label>
                    <input type="text" class="form-control" id="SEOKeyword" placeholder="SEO focus keyword">
                </div>
                <div class="mb-1"><b>Options <span class="small text-danger">Experimental</span></b></div>
                <div class="row">
                    <div class="col">
                        <div class="mb-3">
                            <label for="language" class="form-label">Article Language</label>
                            <select id="language" class="form-select" style="max-width: none;font-size: 1rem;">
                                <option value="Arabic">Arabic</option>
                                <option value="English" selected>English</option>
                                <option value="French">French</option>
                                <option value="German">German</option>
                                <option value="Italian">Italian</option>
                                <option value="Spanish">Spanish</option>
                            </select>
                        </div>
                    </div>
                    <div class="col-2"></div>
                    <div class="col">
                        <label for="media" class="form-label">Include Media (Heatmaps, Stats, etc)</label>
                        <div class="form-check mt-2">
                            <input class="form-check-input" type="checkbox" value="" id="includeMedia">
                        </div>
                    </div>
                </div>
                <div class="mb-1"><b><a href="#" class="advanced-wrapper-toggle">Advanced</a></b></div>
                <div class="advanced-wrapper d-none">
                    <div class="mb-3">
                        <label for="tone" class="form-label">Writing Style</label>
                        <select id="tone" class="form-select" aria-label="Default select example" style="max-width: none;font-size: 1rem;">
                            <option value="Normal" selected>Normal</option>
                            <option value="Straight-Lead Game Story">Straight-Lead Game Story</option>
                            <option value="Feature Game Story">Feature Game Story</option>
                            <option value="Profiles">Profiles</option>
                            <option value="Season Preview and Wrap-up Stories">Season Preview and Wrap-up Stories</option>
                            <option value="Columns">Columns</option>
                        </select>
                    </div>
                    <div class="mb-3">
                        <label for="extraDirectionToDo" class="form-label">Extra direction To Do</label>
                        <textarea class="form-control bg-white" id="extraDirectionToDo" rows="3"></textarea>
                    </div>
                    <div class="mb-3">
                        <label for="extraDirectionToAvoid" class="form-label">Extra direction To Avoid</label>
                        <textarea class="form-control bg-white" id="extraDirectionToAvoid" rows="3"></textarea>
                    </div>
                </div>
                <div class="d-grid gap-2 mt-5">
                    <button type="button" class="btn btn-success d-block generate-button">Generate</button>
                </div>
            </div>
        </div>
        <div id="fw-post-status"></div>
    </div>
    <script type="text/javascript">
        jQuery(document).ready(function($) {
            $('#fw-trigger-form').on('submit', function(e) {
                e.preventDefault();
                var data = $(this).serialize();
                $('#fw-post-status').text('Generating post...');
                $.post(ajaxurl, data, function(response) {
                    $('#fw-post-status').text(response.data.message);
                });
            });
        });
    </script>
    <?php
}


function fw_settings_page() {
    ?>
    <div class="wrap">
        <form method="post" action="options.php">
            <?php
            settings_fields('fw_options_group');
            do_settings_sections('football-writer-settings');
            submit_button('Save Settings');
            ?>
        </form>
    </div>
    <?php
}

function custom_submenu_redirect($current_screen) {
    // Check if we're on the specific submenu page
    if ($current_screen->id === 'football-writer_page_football-writer-create-article') { // The screen ID for the submenu page
        // Redirect to another admin page
        $token = get_option('fw_api_token');
        $secretToken = get_option('fw_api_secret');
        if (!$token || !$secretToken) {
            wp_redirect(admin_url('admin.php?page=football-writer-settings'));
            exit;
        }
    }
}
add_action('current_screen', 'custom_submenu_redirect');
function fw_register_settings() {
    register_setting('fw_options_group', 'fw_api_token', 'sanitize_text_field');
    register_setting('fw_options_group', 'fw_api_secret', 'sanitize_text_field');
    add_settings_section('fw_api_settings', 'Football Writer API Settings', 'fw_api_settings_callback', 'football-writer-settings');
    add_settings_field('fw_api_token', 'API Token', 'fw_api_token_callback', 'football-writer-settings', 'fw_api_settings');
    add_settings_field('fw_api_secret', 'API Secret Token', 'fw_api_secret_token_callback', 'football-writer-settings', 'fw_api_settings');
}

function fw_api_settings_callback() {
    echo '<p>Enter your API configuration below. Provide the Security Token to the API provider to secure webhooks.</p>';
}

function fw_api_token_callback() {
    $token = get_option('fw_api_token');
    echo '<div class="mb-3">';
        echo '<input type="text" class="form-control w-50" name="fw_api_token" id="fw_api_token" placeholder="API Token" value="'.$token.'">';
    echo '</div>';
}

function fw_api_secret_token_callback() {
    $secretToken = get_option('fw_api_secret');
    echo '<div class="mb-3">';
    echo '<input type="text" class="form-control w-50" name="fw_api_secret" id="fw_api_secret" placeholder="API Secret Token" value="'.$secretToken.'">';
    echo '</div>';
}

add_action('admin_init', 'fw_register_settings');

add_action('init', 'fw_listen_for_webhook');

function fw_listen_for_webhook() {
    if (isset($_GET['fw_webhook']) && $_GET['fw_webhook'] == '1' && isset($_GET['wp_post_id'])) {
        $secretToken = get_option('fw_api_secret');
        $data = json_decode(file_get_contents('php://input'));
        $signature = $data->signature;
        $validation = validateHmac($data->dataHash, $signature, $secretToken);
        if ($validation) {
            $parsedown = new Parsedown();
            $htmlContent = $parsedown->text($data->data->markDownContent);
            $post_id = $_GET['wp_post_id'];
            //update post with html content and keep it private and add tags to it
            wp_update_post(array(
                'ID' => $post_id,
                'post_content' => $htmlContent,
                'post_status' => 'private',
                'tags_input' => $data->data->tags
            ));

            //return success
            wp_send_json_success(['message' => 'Post updated successfully.']);
        } else {
            wp_die('Invalid security token.');
        }
    }
}